@@ -23,8 +23,6 @@ use crate::{
     FieldParameters,
     LegendreSymbol,
     One,
-    PoseidonMDSField,
-    PoseidonMDSParameters,
     PrimeField,
     SquareRootField,
     Zero,
@@ -615,30 +613,3 @@ impl<'a, P: Fp256Parameters> DivAssign<&'a Self> for Fp256<P> {
         self.mul_assign(&other.inverse().unwrap());
     }
 }
-
-impl<P: Fp256Parameters + PoseidonMDSParameters> PoseidonMDSField for Fp256<P> {
-    fn poseidon_mds_matrix() -> Vec<Vec<Self>> {
-        let mut mds = Vec::<Vec<Self>>::new();
-        for row in P::POSEIDON_MDS.iter() {
-            mds.push(
-                row.iter()
-                    .map(|b| Self::from_repr_unchecked(*b))
-                    .collect::<Vec<Self>>()
-                    .to_vec(),
-            );
-        }
-        mds
-    }
-
-    fn poseidon_alpha() -> u64 {
-        P::POSEIDON_ALPHA
-    }
-
-    fn poseidon_number_full_rounds() -> u32 {
-        P::POSEIDON_FULL_ROUNDS
-    }
-
-    fn poseidon_number_partial_rounds() -> u32 {
-        P::POSEIDON_PARTIAL_ROUNDS
-    }
-}
