@@ -23,33 +23,28 @@ macro_rules! field {
         }
     };
     ($name:ident, $c0:expr, $c1:expr $(,)?) => {
-        $name {
-            c0: $c0,
-            c1: $c1,
-            _parameters: std::marker::PhantomData,
-        }
+        $name { c0: $c0, c1: $c1 }
     };
     ($name:ident, $c0:expr, $c1:expr, $c2:expr $(,)?) => {
         $name {
             c0: $c0,
             c1: $c1,
             c2: $c2,
-            _parameters: std::marker::PhantomData,
         }
     };
 }
 
-macro_rules! impl_field_into_bigint {
-    ($field: ident, $bigint: ident, $params: ident) => {
-        impl<P: $params> Into<$bigint> for $field<P> {
-            fn into(self) -> $bigint {
-                self.into_repr()
+macro_rules! impl_field_into_biginteger {
+    ($field: ident, $biginteger: ident, $parameters: ident) => {
+        impl<P: $parameters> Into<$biginteger> for $field<P> {
+            fn into(self) -> $biginteger {
+                self.to_repr()
             }
         }
     };
 }
 
-macro_rules! impl_prime_field_standard_sample {
+macro_rules! impl_primefield_standard_sample {
     ($field: ident, $params: ident) => {
         impl<P: $params> rand::distributions::Distribution<$field<P>> for rand::distributions::Standard {
             #[inline]
@@ -71,7 +66,7 @@ macro_rules! impl_prime_field_standard_sample {
     };
 }
 
-macro_rules! impl_prime_field_from_int {
+macro_rules! impl_primefield_from_int {
     ($field: ident, u128, $params: ident) => {
         impl<P: $params> From<u128> for $field<P> {
             /// Attempts to convert an integer into a field element.
@@ -154,11 +149,11 @@ macro_rules! sqrt_impl {
     }};
 }
 
-macro_rules! impl_prime_field_serializer {
+macro_rules! impl_primefield_serializer {
     ($field: ident, $params: ident, $byte_size: expr) => {
         impl<P: $params> CanonicalSerializeWithFlags for $field<P> {
             #[allow(unused_qualifications)]
-            fn serialize_with_flags<W: snarkvm_utilities::io::Write, F: snarkvm_utilities::serialize::Flags>(
+            fn serialize_with_flags<W: snarkvm_utilities::io::Write, F: snarkvm_utilities::Flags>(
                 &self,
                 writer: &mut W,
                 flags: F,
@@ -166,13 +161,13 @@ macro_rules! impl_prime_field_serializer {
                 const BYTE_SIZE: usize = $byte_size;
 
                 let (output_bit_size, output_byte_size) =
-                    snarkvm_utilities::serialize::buffer_bit_byte_size($field::<P>::size_in_bits());
-                if F::len() > (output_bit_size - P::MODULUS_BITS as usize) {
+                    snarkvm_utilities::serialize::number_of_bits_and_bytes($field::<P>::size_in_bits());
+                if F::num_bits() > (output_bit_size - P::MODULUS_BITS as usize) {
                     return Err(snarkvm_utilities::errors::SerializationError::NotEnoughSpace);
                 }
 
                 let mut bytes = [0u8; BYTE_SIZE];
-                self.write(&mut bytes[..])?;
+                self.write_le(&mut bytes[..])?;
 
                 bytes[output_byte_size - 1] |= flags.u8_bitmask();
 
@@ -182,7 +177,7 @@ macro_rules! impl_prime_field_serializer {
         }
 
         impl<P: $params> ConstantSerializedSize for $field<P> {
-            const SERIALIZED_SIZE: usize = snarkvm_utilities::serialize::buffer_byte_size(
+            const SERIALIZED_SIZE: usize = snarkvm_utilities::serialize::number_of_bits_to_number_of_bytes(
                 <$field<P> as crate::PrimeField>::Parameters::MODULUS_BITS as usize,
             );
             const UNCOMPRESSED_SIZE: usize = Self::SERIALIZED_SIZE;
@@ -206,14 +201,14 @@ macro_rules! impl_prime_field_serializer {
 
         impl<P: $params> CanonicalDeserializeWithFlags for $field<P> {
             #[allow(unused_qualifications)]
-            fn deserialize_with_flags<R: snarkvm_utilities::io::Read, F: snarkvm_utilities::serialize::Flags>(
+            fn deserialize_with_flags<R: snarkvm_utilities::io::Read, F: snarkvm_utilities::Flags>(
                 reader: &mut R,
             ) -> Result<(Self, F), snarkvm_utilities::errors::SerializationError> {
                 const BYTE_SIZE: usize = $byte_size;
 
                 let (output_bit_size, output_byte_size) =
-                    snarkvm_utilities::serialize::buffer_bit_byte_size($field::<P>::size_in_bits());
-                if F::len() > (output_bit_size - P::MODULUS_BITS as usize) {
+                    snarkvm_utilities::serialize::number_of_bits_and_bytes($field::<P>::size_in_bits());
+                if F::num_bits() > (output_bit_size - P::MODULUS_BITS as usize) {
                     return Err(snarkvm_utilities::errors::SerializationError::NotEnoughSpace);
                 }
 
@@ -222,7 +217,7 @@ macro_rules! impl_prime_field_serializer {
 
                 let flags = F::from_u8_remove_flags(&mut masked_bytes[output_byte_size - 1]);
 
-                Ok((Self::read(&masked_bytes[..])?, flags))
+                Ok((Self::read_le(&masked_bytes[..])?, flags))
             }
         }
 
@@ -230,15 +225,15 @@ macro_rules! impl_prime_field_serializer {
             #[allow(unused_qualifications)]
             fn deserialize<R: snarkvm_utilities::io::Read>(
                 reader: &mut R,
-            ) -> Result<Self, snarkvm_utilities::errors::SerializationError> {
+            ) -> Result<Self, snarkvm_utilities::SerializationError> {
                 const BYTE_SIZE: usize = $byte_size;
 
                 let (_, output_byte_size) =
-                    snarkvm_utilities::serialize::buffer_bit_byte_size($field::<P>::size_in_bits());
+                    snarkvm_utilities::serialize::number_of_bits_and_bytes($field::<P>::size_in_bits());
 
                 let mut masked_bytes = [0; BYTE_SIZE];
                 reader.read_exact(&mut masked_bytes[..output_byte_size])?;
-                Ok(Self::read(&masked_bytes[..])?)
+                Ok(Self::read_le(&masked_bytes[..])?)
             }
         }
 
@@ -301,38 +296,56 @@ macro_rules! impl_prime_field_serializer {
 }
 
 macro_rules! impl_field_from_random_bytes_with_flags {
-    ($limbs: expr) => {
+    ($u64_limbs: expr) => {
         #[inline]
-        fn from_random_bytes_with_flags(bytes: &[u8]) -> Option<(Self, u8)> {
-            let mut result_bytes = [0u8; $limbs * 8];
-            for (result_byte, in_byte) in result_bytes.iter_mut().zip(bytes.iter()) {
-                *result_byte = *in_byte;
-            }
+        fn from_random_bytes_with_flags<F: snarkvm_utilities::Flags>(bytes: &[u8]) -> Option<(Self, F)> {
+            // Copy the input into a temporary buffer.
+            let mut result_bytes = [0u8; $u64_limbs * 8 + 1];
+            result_bytes.iter_mut().zip(bytes).for_each(|(result, input)| {
+                *result = *input;
+            });
 
-            let mask: u64 = 0xffffffffffffffff >> P::REPR_SHAVE_BITS;
-            // the flags will be at the same byte with the lowest shaven bits or the one after
-            let flags_byte_position: usize = 7 - P::REPR_SHAVE_BITS as usize / 8;
-            let flags_mask: u8 = ((1 << P::REPR_SHAVE_BITS % 8) - 1) << (8 - P::REPR_SHAVE_BITS % 8);
-            // take the last 8 bytes and pass the mask
-            let last_bytes = &mut result_bytes[($limbs - 1) * 8..];
+            // The `last_limb_mask` retains everything in the final limb up to `P::MODULUS_BITS`.
+            let last_limb_mask = u64::MAX >> P::REPR_SHAVE_BITS;
+
+            let mut last_bytes_mask = [0u8; 9];
+            last_bytes_mask[..8].copy_from_slice(&last_limb_mask.to_le_bytes());
+
+            // Length of the buffer containing the field element and the flag.
+            let output_byte_size =
+                snarkvm_utilities::number_of_bits_to_number_of_bytes(P::MODULUS_BITS as usize + F::num_bits());
+
+            // Flags are located in the final byte of the serialized field representation.
+            let flag_location = output_byte_size - 1;
+
+            // At which byte is the flag located in the last limb?
+            let flag_location_in_last_limb = flag_location - (8 * ($u64_limbs - 1));
+
+            // Take all but the last 9 bytes.
+            let last_bytes = &mut result_bytes[8 * ($u64_limbs - 1)..];
+
+            // The mask only has the last `F::num_bits()` bits set.
+            let flags_mask = u8::MAX.checked_shl(8 - (F::num_bits() as u32)).unwrap_or(0);
+
+            // Mask away the remaining bytes, and try to reconstruct the flag.
             let mut flags: u8 = 0;
-            for (i, (b, m)) in last_bytes.iter_mut().zip(&mask.to_le_bytes()).enumerate() {
-                if i == flags_byte_position {
+            for (i, (b, m)) in last_bytes.iter_mut().zip(&last_bytes_mask).enumerate() {
+                if i == flag_location_in_last_limb {
                     flags = *b & flags_mask
                 }
                 *b &= m;
             }
 
-            <Self as CanonicalDeserialize>::deserialize(&mut &result_bytes[..])
+            <Self as CanonicalDeserialize>::deserialize(&mut &result_bytes[..($u64_limbs * 8)])
                 .ok()
-                .map(|f| (f, flags))
+                .map(|f| (f, F::from_u8(flags)))
         }
     };
 }
 
-// Implements AddAssign on Self by deferring to an implementation on &Self
+/// Implements Add, Sub, AddAssign, and SubAssign on Self by deferring to an implementation on &Self
 #[macro_export]
-macro_rules! impl_additive_ops_from_ref {
+macro_rules! impl_add_sub_from_field_ref {
     ($type: ident, $params: ident) => {
         #[allow(unused_qualifications)]
         impl<P: $params> core::ops::Add<Self> for $type<P> {
@@ -464,9 +477,9 @@ macro_rules! impl_additive_ops_from_ref {
     };
 }
 
-// Implements AddAssign on Self by deferring to an implementation on &Self
+/// Implements Mul, Div, MulAssign, and DivAssign on Self by deferring to an implementation on &Self
 #[macro_export]
-macro_rules! impl_multiplicative_ops_from_ref {
+macro_rules! impl_mul_div_from_field_ref {
     ($type: ident, $params: ident) => {
         #[allow(unused_qualifications)]
         impl<P: $params> core::ops::Mul<Self> for $type<P> {
