@@ -40,6 +40,3 @@ pub use to_constraint_field::*;
 
 mod zero;
 pub use zero::*;
-
-mod poseidon_mds_field;
-pub use poseidon_mds_field::*;
